@@ -203,7 +203,7 @@ namespace Beagle.Daemon {
 				if (! indexable.NoContent) {
 					indexable.NoContent = true;
 
-					Logger.Log.Debug ("No filter for {0} ({1})", path != null ? path : indexable.Uri.ToString (), indexable.MimeType);
+					Logger.Log.Debug ("No filter for {0} ({1}) [{2}]", indexable.Uri, path, indexable.MimeType);
 					return false;
 				}
 				
@@ -335,11 +335,11 @@ namespace Beagle.Daemon {
 		[XmlAttribute ("Uri")]
 		public string UriAsString {
 			get {
-				return UriFu.UriToSerializableString (uri);
+				return UriFu.UriToEscapedString (uri);
 			}
 
 			set {
-				uri = UriFu.UriStringToUri (value);
+				uri = UriFu.EscapedStringToUri (value);
 			}
 		}
 
