@@ -74,7 +74,9 @@ namespace Beagle.Daemon {
 		//     date properties
 		// 13: moved source into a property
 		// 14: allow wildcard queries to also match keywords
-		private const int MAJOR_VERSION = 14;
+		// 15: analyze PropertyKeyword field, and store all properties as
+		//     lower case so that we're truly case insensitive.
+		private const int MAJOR_VERSION = 15;
 		private int minor_version = 0;
 
 		private string index_name;
@@ -404,9 +406,19 @@ namespace Beagle.Daemon {
 					// If this is non-text property, just return one token
 					// containing the entire string.  We do this to avoid
 					// tokenizing keywords.
-					if (! is_text_prop)
-						return new SingletonTokenStream (reader.ReadToEnd ());
-				}
+					if (! is_text_prop) {
+						// We don't want to lower case the token if it's
+						// not in the private namespace.
+							
+						TokenStream singleton_stream = new SingletonTokenStream (reader.ReadToEnd ());
+						
+						if (fieldName.StartsWith ("prop:k:" + LuceneQueryingDriver.PrivateNamespace))
+							return singleton_stream;
+						else
+							return new LowerCaseFilter (singleton_stream);
+					}
+				} else if (fieldName == "PropertyKeyword")
+					return new LowerCaseFilter (new SingletonTokenStream (reader.ReadToEnd ()));
 
 				TokenStream outstream;
 				outstream = base.TokenStream (fieldName, reader);
@@ -508,13 +520,12 @@ namespace Beagle.Daemon {
 
 			if (prop.IsSearched) {
 				string wildcard_field = TypeToWildcardField (prop.Type);
-				bool tokenize = (prop.Type == PropertyType.Text);
 				if (wildcard_field != null) {
 					f = new Field (wildcard_field,
 						       prop.Value,
 						       false, // never stored
 						       true,  // always indexed
-						       tokenize);
+						       true); // always tokenize (just lowercases for keywords; full analysis for text)
 					doc.Add (f);
 
 					if (prop.Type == PropertyType.Date)
@@ -533,7 +544,7 @@ namespace Beagle.Daemon {
 				       coded_value,
 				       prop.IsStored,
 				       true,        // always index
-				       true);       // always tokenize (just strips off type code for keywords)
+				       true);       // always tokenize (strips off type code for keywords and lowercases)
 			doc.Add (f);
 
 			if (prop.Type == PropertyType.Date)
@@ -586,11 +597,11 @@ namespace Beagle.Daemon {
 
 			Field f;
 
-			f = Field.Keyword ("Uri", UriFu.UriToSerializableString (indexable.Uri));
+			f = Field.Keyword ("Uri", UriFu.UriToEscapedString (indexable.Uri));
 			primary_doc.Add (f);
 
 			if (indexable.ParentUri != null) {
-				f = Field.Keyword ("ParentUri", UriFu.UriToSerializableString (indexable.ParentUri));
+				f = Field.Keyword ("ParentUri", UriFu.UriToEscapedString (indexable.ParentUri));
 				primary_doc.Add (f);
 			}
 			
@@ -675,7 +686,7 @@ namespace Beagle.Daemon {
 				if (prop.IsMutable) {
 					if (secondary_doc == null) {
 						secondary_doc = new Document ();
-						f = Field.Keyword ("Uri", UriFu.UriToSerializableString (indexable.Uri));
+						f = Field.Keyword ("Uri", UriFu.UriToEscapedString (indexable.Uri));
 						secondary_doc.Add (f);
 					}
 					target_doc = secondary_doc;
@@ -695,7 +706,7 @@ namespace Beagle.Daemon {
 			new_doc = new Document ();
 
 			Field uri_f;
-			uri_f = Field.Keyword ("Uri", UriFu.UriToSerializableString (prop_only_indexable.Uri));
+			uri_f = Field.Keyword ("Uri", UriFu.UriToEscapedString (prop_only_indexable.Uri));
 			new_doc.Add (uri_f);
 
 			Logger.Log.Debug ("Rewriting {0}", prop_only_indexable.DisplayUri);
@@ -734,7 +745,7 @@ namespace Beagle.Daemon {
 			uri = doc.Get ("Uri");
 			if (uri == null)
 				throw new Exception ("Got document from Lucene w/o a URI!");
-			return UriFu.UriStringToUri (uri);
+			return UriFu.EscapedStringToUri (uri);
 		}
 
 		static protected Hit DocumentToHit (Document doc)
@@ -747,7 +758,7 @@ namespace Beagle.Daemon {
 			string str;
 			str = doc.Get ("ParentUri");
 			if (str != null)
-				hit.ParentUri = UriFu.UriStringToUri (str);
+				hit.ParentUri = UriFu.EscapedStringToUri (str);
 			
 			hit.Timestamp = StringFu.StringToDateTime (doc.Get ("Timestamp"));
 
@@ -1211,7 +1222,7 @@ namespace Beagle.Daemon {
 					}
 
 					Term term;
-					term = new Term ("PropertyKeyword", part.Text);
+					term = new Term ("PropertyKeyword", part.Text.ToLower ()); // make sure text is lowercased
 					// FIXME: terms are already added in term_list. But they may have been tokenized
 					// The term here is non-tokenized version. Should this be added to term_list ?
 					// term_list is used to calculate scores
@@ -1231,6 +1242,49 @@ namespace Beagle.Daemon {
 				return;
 			}
 
+			if (abstract_part is QueryPart_Wildcard) {
+				QueryPart_Wildcard part = (QueryPart_Wildcard) abstract_part;
+
+				LNS.BooleanQuery p_query = new LNS.BooleanQuery ();
+				LNS.BooleanQuery s_query = new LNS.BooleanQuery ();
+				
+				Term term;
+				LNS.Query subquery;
+
+				// Lower case the terms for searching
+				string query_string_lower = part.QueryString.ToLower ();
+
+				// Search text content
+				term = new Term ("Text", query_string_lower);
+				subquery = new LNS.WildcardQuery (term);
+				p_query.Add (subquery, false, false);
+				term_list.Add (term);
+
+				// Search text properties
+				term = new Term ("PropertyText", query_string_lower);
+				subquery = new LNS.WildcardQuery (term);
+				p_query.Add (subquery, false, false);
+				// Properties can live in either index
+				if (! only_build_primary_query)
+					s_query.Add (subquery.Clone () as LNS.Query, false, false);
+				term_list.Add (term);
+
+				// Search property keywords
+				term = new Term ("PropertyKeyword", query_string_lower);
+				term_list.Add (term);
+				subquery = new LNS.WildcardQuery (term);
+				p_query.Add (subquery, false, false);
+				// Properties can live in either index
+				if (! only_build_primary_query)
+					s_query.Add (subquery.Clone () as LNS.Query, false, false);
+
+				primary_query = p_query;
+				if (! only_build_primary_query)
+					secondary_query = s_query;
+
+				return;
+			}
+
 			if (abstract_part is QueryPart_Property) {
 				QueryPart_Property part = (QueryPart_Property) abstract_part;
 
@@ -1247,7 +1301,7 @@ namespace Beagle.Daemon {
 					primary_query = StringToQuery (field_name, part.Value, term_list);
 				else {
 					Term term;
-					term = new Term (field_name, part.Value);
+					term = new Term (field_name, part.Value.ToLower ());
 					if (term_list != null)
 						term_list.Add (term);
 					primary_query = new LNS.TermQuery (term);
@@ -1326,7 +1380,7 @@ namespace Beagle.Daemon {
 
 		static protected LNS.Query UriQuery (string field_name, Uri uri)
 		{
-			return new LNS.TermQuery (new Term (field_name, UriFu.UriToSerializableString (uri)));
+			return new LNS.TermQuery (new Term (field_name, UriFu.UriToEscapedString (uri)));
 		}
 
 		static protected LNS.Query UriQuery (string field_name, ICollection uri_list)
